/*
 * modified for lecture by Fabian Fuchs (KIT, Karlsruhe) based on:
 * 
 Copyright (c) 2007, Distributed Computing Group (DCG)
                    ETH Zurich
                    Switzerland
                    dcg.ethz.ch

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

 - Neither the name 'Sinalgo' nor the names of its contributors may be
   used to endorse or promote products derived from this software
   without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package projects.localbroadcast.nodes.nodeImplementations;


import java.awt.Color;
import java.awt.Graphics;
import java.util.HashSet;

import projects.defaultProject.nodes.messages.DefaultMessage;
import projects.localbroadcast.CustomGlobal;
import sinalgo.configuration.WrongConfigurationException;
import sinalgo.gui.transformation.PositionTransformation;
import sinalgo.nodes.Node;
import sinalgo.nodes.messages.Inbox;
import sinalgo.nodes.messages.Message;
import sinalgo.tools.Tools;



/**
 * A Local Broadcast Node - does not work with mobility
 */
public class LocalBroadcastNode extends Node {

	/* store the current round */
	private int round;

	/* Set of neighbors that were successful at reaching me */
	protected HashSet<Integer> neighborsThatReachedMe;
	
	/* flag that indicates that the node was reached by all its neighbors
	 * if this flag is set, the node notifies CustomGlobal that increases a global counter
	 * */
	private boolean reachedByAllNeighbors;
	
	/* number of neighbors of the node */
	private int numNeighbors;
	
	/* transmission probability of the node*/
	private double txProbability;

	/* payload for message */
	public class LBPayload {
		public final int fromID;
		private LBPayload(int id) {
			fromID = id;
		}
	}
	
	@Override
	public void handleMessages(Inbox inbox) {
		while(inbox.hasNext()) {
			Message msg = inbox.next();
			if(msg instanceof DefaultMessage<?>) {
				//todo: Handle message
			}
		}

		/*
		 * notify the global mananger if all nodes reached me? 
		 * Can be done here or at many other places
		 * ((CustomGlobal)Tools.getCustomGlobal()).notifyReachedByAllNeigbhors(this.ID);
		 */
	}
	

	private boolean trueWithProbability(double prob) {
		return (Tools.getRandomNumberGenerator().nextDouble() < prob);
	}

	@Override
	public void preStep() {
		//executed before each 'step' (i.e. timers and handleMessages)
		//I would personally recommend to only use the postStep method for simplicity
	}
	
	@Override
	public void postStep() {
		//executed after each 'step' (i.e. timers and handleMessages)
		round++;
		
		/* 
		 * init transmission probability
		 */
		if (round == 1) {
			double txConstant = Tools.getConfigurationDouble("SINRTransmission/constant", 
					"The configuration entry SINRTransmission/constant is not a valid double");
			txProbability = 1 / (txConstant * (numNeighbors + 1));
		}
		
		//todo: transmit with certain probability in each round/time slot
		
	}

	@Override
	public void init() {
		// initialize the node / read configuration values
		round = 0;
		reachedByAllNeighbors = false;
		neighborsThatReachedMe = new HashSet<Integer>();
		
		//do not init tx Probability here, this method is called during node creation!
	}
	
	
	@Override
	public void neighborhoodChange() {
		//todo: set number of neighbors
		
	}
		
	/* (non-Javadoc)
	 * @see sinalgo.nodes.Node#draw(java.awt.Graphics, sinalgo.gui.transformation.PositionTransformation, boolean)
	 */
	public void draw(Graphics g, PositionTransformation pt, boolean highlight) {
		//set the color of this node, this allows us to see how the most important maxIDs spread
		

		this.setColor(Color.LIGHT_GRAY);
		if (reachedByAllNeighbors)
			this.setColor(Color.GREEN);
		
		String text = Integer.toString(this.ID);
		// draw the node as a circle with the text inside
		super.drawNodeAsDiskWithText(g, pt, highlight, text, 10, Color.YELLOW);
	}


	@Override
	public String toString() {
		return "numNeighbors: " + numNeighbors; 
	}

	@Override
	public void checkRequirements() throws WrongConfigurationException {
		//does configuration fit the nodes requirements? 
		
	}
}
