/*
 * modified for lecture by Fabian Fuchs (KIT, Karlsruhe) based on:
 * 
 Copyright (c) 2007, Distributed Computing Group (DCG)
                    ETH Zurich
                    Switzerland
                    dcg.ethz.ch

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

 - Neither the name 'Sinalgo' nor the names of its contributors may be
   used to endorse or promote products derived from this software
   without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package projects.randcolor.nodes.nodeImplementations;


import java.awt.Color;
import java.awt.Graphics;
import java.util.HashSet;

import projects.defaultProject.nodes.messages.DefaultMessage;
import projects.localbroadcast.CustomGlobal;
import sinalgo.configuration.WrongConfigurationException;
import sinalgo.gui.transformation.PositionTransformation;
import sinalgo.nodes.Node;
import sinalgo.nodes.edges.Edge;
import sinalgo.nodes.messages.Inbox;
import sinalgo.nodes.messages.Message;
import sinalgo.tools.Tools;



/**
 * A Local Broadcast Node - does not work with mobility
 */
public class RandColorNode extends Node {

	/* store the current round */
	private int round;

	/* number of rounds between the node checks for conflicsts */
	private int lengthOfPhase;
	
	/* indicated the next round that the node should be active,   
	 * check whether a conflict was detected and if so,
	 * select a new random color
	 */
	private int nextActiveRound;
		
	/* number of neighbors of the node */
	private int numNeighbors;
	
	/* transmission probability of the node*/
	private double txProbability;
	
	/* current color of the node */
	private int color;
	
	/* flag that indicates whether a conflict was detected*/
	private boolean conflictDetected;
	
	public int getCurrentColor() {
		return this.color;
	}

	/* payload for message */
	public class RCPayload {
		public final int color;
		private RCPayload(int c) {
			color = c;
		}
	}
	
	@Override
	public void handleMessages(Inbox inbox) {
		while(inbox.hasNext()) {
			Message msg = inbox.next();
			if(msg instanceof DefaultMessage<?>) {
				if (((DefaultMessage<?>) msg).getLoad() instanceof RCPayload) {
					//todo: Handle message
				}
			}
		}
	}
	

	private boolean trueWithProbability(double prob) {
		return (Tools.getRandomNumberGenerator().nextDouble() < prob);
	}

	@Override
	public void preStep() {
		//executed before each 'step' (i.e. timers and handleMessages)
		//I would personally recommend to only use the postStep method for simplicity
	}
	
	/*
	 * selects a random color
	 */
	private void selectNewRandomColor() {
		/* select a new random color using Tools.getRandomNumberGenerator().nextInt
		 * from the set {0,..,4*numNeighbors+1} (+1 ensures that 0 can always be chosen)
		 */
	}
	
	@Override
	public void postStep() {
		//executed after each 'step' (i.e. timers and handleMessages)
		round++;
		
		/* 
		 * init transmission probability 
		 */
		if (round == 1) {
			double txConstant = Tools.getConfigurationDouble("SINRTransmission/constant", 
					"The configuration entry SINRTransmission/constant is not a valid double");
			txProbability = 1 / (txConstant * numNeighbors);
		}
		
		//todo: transmit, check if round is active, handle active round, etc.
	}

	@Override
	public void init() {
		// initialize the node / read configuration values
		round = 0;
		lengthOfPhase = Tools.getConfigurationInteger("RandColor/lengthOfPhase", 
				"The configuration entry RandColor/lengthOfPhase is not a valid integer");
		nextActiveRound = 1;
		
		//do not init tx Probability here, this method is called during node creation!
		//do not init color here, neighbors are not yet established. But do it somewhere :)
	}
	
	
	@Override
	public void neighborhoodChange() {
		//set number of neighbors
		numNeighbors = this.outgoingConnections.size();
	}
		
	/* (non-Javadoc)
	 * @see sinalgo.nodes.Node#draw(java.awt.Graphics, sinalgo.gui.transformation.PositionTransformation, boolean)
	 */
	public void draw(Graphics g, PositionTransformation pt, boolean highlight) {
		//set the color of this node, this allows us to see how the most important maxIDs spread
		

		this.setColor(Color.LIGHT_GRAY);
		for (Edge e : this.outgoingConnections) {
			if ((((RandColorNode)(e.endNode)).getCurrentColor()) == color) {
				this.setColor(Color.RED);
			}
		}
		
		String text = Integer.toString(this.ID);
		// draw the node as a circle with the text inside
		super.drawNodeAsDiskWithText(g, pt, highlight, text, 10, Color.YELLOW);
	}


	@Override
	public String toString() {
		return "color: " + color; 
	}

	@Override
	public void checkRequirements() throws WrongConfigurationException {
		//does configuration fit the nodes requirements? 
		
	}
}
